﻿using Infragistics.Controls.Charts;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Windows.Threading;

namespace WpfApplication1
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        FamilyTreeViewModel viewmodel = new FamilyTreeViewModel();
        public MainWindow()
        {
            InitializeComponent();
        }

        private void Diagram_SelectionChanged(object sender, Infragistics.Controls.Charts.DiagramSelectionChangedEventArgs e)
        {                        
            if (e.AddedItems.Count() > 0 && e.AddedItems[0] is DiagramItem)
            {
                ObservableCollection<Person> personCollection = viewmodel.FamilyTree;

                Person item = e.AddedItems[0].Content as Person;

                DiagramToolboxCategoryCollection customCategoryCollection = new DiagramToolboxCategoryCollection();
               
                DiagramToolboxCategory Parents = new DiagramToolboxCategory();
                if (item != null)
                {
                    if (item.Relation == "Mother" || item.Relation == "Father")
                    {
                        customCategoryCollection.Add(Parents);
                        toolbox.CustomCategories.Add(customCategoryCollection[0]);
                        DiagramToolboxItem toolboxItem = new DiagramToolboxItem();

                        customCategoryCollection[0].Items.Add(toolboxItem);
                        DiagramConnection connection = new DiagramConnection();
                        connection.ConnectionType = DiagramConnectionType.Straight;
                        connection.StartPosition = new Point(10, 10);
                        connection.EndPosition = new Point(40, 70);
                        connection.EndCapType = DiagramConnectionCapType.FilledArrow;

                        toolboxItem.Item = connection;
                    }
                    else
                    {
                        customCategoryCollection.Clear();
                        if (toolbox.CustomCategories.Count() > 0)
                        {
                            toolbox.CustomCategories.RemoveAt(0);
                        }

                    }
                                     
                } 
                       
            }            
        }

        private void toolbox_Drop(object sender, DragEventArgs e)
        {

        }

        private void Diagram_ItemAdded(object sender, DiagramItemAddedEventArgs e)
        {
            DiagramConnection item = e.Item as DiagramConnection;

            if (item != null)
            {
                //Add to connection to selected DiagramNode, if one is already selected.
                if (Diagram.SelectedItems[0] is DiagramNode)
                {
                    var parameters = new object[] { item, this.Diagram.SelectedItems[0], this.Diagram.Items[0] };

                    Action<DiagramConnection, DiagramNode, DiagramNode> action = (con, startNode, endNode) =>
                    {
                        con.StartNode = startNode;
                        con.StartNodeConnectionPointName = startNode.ConnectionPoints[3].Name;

                        con.EndNode = endNode;
                        con.EndNodeConnectionPointName = endNode.ConnectionPoints[3].Name;
                    };

                    // Call this asynchronously to prevent the default handling of a newly added item from resetting the start and end node.
                    this.Diagram.Dispatcher.BeginInvoke(action, DispatcherPriority.Background, parameters);
                }
            }
        }

        private void Diagram_ConnectionConnected(object sender, DiagramConnectionConnectedEventArgs e)
        {
            //Add connection to parent object
        }
    }


    public class FamilyTreeViewModel : INotifyPropertyChanged
    {
        ObservableCollection<Person> _familyTree;
        public ObservableCollection<Person> FamilyTree
        {
            get { return _familyTree; }
            set
            {
                _familyTree = value;
                OnPropertyChanged("FamilyTree");
                _familyTree.CollectionChanged += _familyTree_CollectionChanged;
            }
        }

        private void _familyTree_CollectionChanged(object sender, System.Collections.Specialized.NotifyCollectionChangedEventArgs e)
        {
          
        }

        public FamilyTreeViewModel()
        {
            var me = new Person(1, "Me", "John", "GUY04.jpg");
            var dad = new Person(2, "Father", "James", "GUY20.jpg") { Children = new List<int>() { 1 } };
            var mom = new Person(3, "Mother", "Jane", "GIRL15.jpg") { Children = new List<int>() { 1 } };
            //var grandpa = new Person(4, DiagramStrings.FamiliyTree_Grandfather, DiagramStrings.FamilyTree_Steve, "GUY19.jpg") { Children = new List<int>() { 2 } };
            //var grandma = new Person(5, DiagramStrings.FamiliyTree_Grandfather, DiagramStrings.FamilyTree_William, "GUY05.jpg") { Children = new List<int>() { 3 } };

            //var wife = new Person(6, DiagramStrings.FamiliyTree_Wife, DiagramStrings.FamilyTree_Christine, "GIRL28.jpg") { Children = new List<int>() { 1 } };
            //var herdad = new Person(7, DiagramStrings.FamiliyTree_Father, DiagramStrings.FamilyTree_Charles, "GUY01.jpg") { Children = new List<int>() { 6 } };
            //var hermom = new Person(8, DiagramStrings.FamiliyTree_Mother, DiagramStrings.FamilyTree_Maria, "GIRL29.jpg") { Children = new List<int>() { 6 } };
            //var hergrandpa = new Person(9, DiagramStrings.FamiliyTree_Grandfather, DiagramStrings.FamilyTree_Bob, "GUY09.jpg") { Children = new List<int>() { 7 } };
            //var hergrandma = new Person(10, DiagramStrings.FamiliyTree_Grandmother, DiagramStrings.FamilyTree_Eve, "GIRL22.jpg") { Children = new List<int>() { 7 } };

            FamilyTree = new ObservableCollection<Person>();
            FamilyTree.Add(me);
            FamilyTree.Add(dad);
            FamilyTree.Add(mom);
            //FamilyTree.Add(grandpa);
            //FamilyTree.Add(grandma);
            //FamilyTree.Add(wife);
            //FamilyTree.Add(herdad);
            //FamilyTree.Add(hermom);
            //FamilyTree.Add(hergrandpa);
            //FamilyTree.Add(hergrandma);
        }

        public event PropertyChangedEventHandler PropertyChanged;
        void OnPropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }
    }

    public class Person : INotifyPropertyChanged
    {
        public Person(int id, string relation, string name, string path)
        {
            Id = id;
            Relation = relation;
            Name = name;
            //ImagePath = ImageFilePathProvider.GetImageLocalPath() + "people/" + path;
        }

        int _id;
        public int Id
        {
            get { return _id; }
            set
            {
                _id = value;
                OnPropertyChanged("Id");
            }
        }

        string _name;
        public string Name
        {
            get { return _name; }
            set
            {
                _name = value;
                OnPropertyChanged("Name");
            }
        }

        string _relation;
        public string Relation
        {
            get { return _relation; }
            set
            {
                _relation = value;
                OnPropertyChanged("Relation");
            }
        }

        string _path;
        public string ImagePath
        {
            get { return _path; }
            set
            {
                _path = value;
                OnPropertyChanged("ImagePath");
            }
        }

        IList _children;
        public IList Children
        {
            get { return _children; }
            set
            {
                _children = value;
                OnPropertyChanged("Children");
            }
        }

        public event PropertyChangedEventHandler PropertyChanged;
        void OnPropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }
    }
}
